;; Generated package description from geiser-mit.el  -*- no-byte-compile: t -*-
(define-package "geiser-mit" "0.15.0.20240909.114537" "MIT/GNU Scheme's implementation of the geiser protocols" '((emacs "24.4") (geiser "0.18")) :commit "ddd2ba733e8274d40a26b5d6d2ee11f1bac8abe6" :authors '(("Peter" . "craven@gmx.net")) :maintainer '("Jose A Ortega Ruiz" . "jao@gnu.org") :keywords '("languages" "mit" "scheme" "geiser") :url "https://gitlab.com/emacs-geiser/mit")
